@@ -1,13 +1,15 @@
 
-::oo::class create ::loon::classes::LayerVisual { 
-    
+::oo::class create ::loon::classes::LayerVisual {
+
     superclass ::loon::classes::Visual
-    
+
     variable layerobj statebinding container tag_var x_var y_var
-    
+
     ## layerid is visualid
     constructor {Layerobj Container args} {
-	
+
+    set classes [info object class $Layerobj]
+
 	set layerobj $Layerobj
 	set statebinding ""
 	set container $Container
@@ -18,30 +20,30 @@
 	} else {
 	    set ns [info object namespace $Layerobj]
 	}
-	
+
 	foreach state {x y tag} {
 	    set ${state}_var [uplevel #0 [list ${ns}::my varname $state]]
 	}
-	
+
 	my MakeStateBinding
-	
+
 	next {*}$args
-	
+
     }
-    
+
     method MakeStateBinding {} {
       	set statebinding [$layerobj systembind state add all "[self] layerchanged %e"]
     }
 
-    destructor {	
+    destructor {
 	if {$statebinding ne ""} {
 	    ## destroy gets called if the window gets cloesed
-	    ## or the layer object gets deleted	    
+	    ## or the layer object gets deleted
 	    catch {$layerobj systembind state delete $statebinding}
 	}
 	next
     }
-    
+
     method layerchanged {events} {
 	if {$events eq "destroy"} {
 	    my destroy
@@ -51,16 +53,15 @@
     }
     method layerupdate {events} {
 	my redraw
-    }   
+    }
+
 
-    
     method InfoDebug {args} {
 	next layerobj statebinding {*}$args
     }
 
     method redraw {} {
 	my variable visualid
-	
 	## visualid is also the layer id
 	$container layerUpdate $visualid move
     }
@@ -68,5 +69,5 @@
     method updateZoomPan {oldPanX oldPanY oldZoomX oldZoomY} {
 	my updateCoords
     }
-    
+
 }
